@@ -1,12 +1,15 @@
-use std::fmt;
-use std::ops::{
+use core::fmt;
+use core::ops::{
     BitAnd, BitAndAssign, BitOr, BitOrAssign, BitXor, BitXorAssign, RangeInclusive, Sub, SubAssign,
 };
 
 use super::store::{self, Store};
 use super::util;
 
-const ARRAY_LIMIT: u64 = 4096;
+pub const ARRAY_LIMIT: u64 = 4096;
+
+#[cfg(not(feature = "std"))]
+use alloc::vec::Vec;
 
 #[derive(PartialEq, Clone)]
 pub struct Container {
@@ -14,6 +17,7 @@ pub struct Container {
     pub store: Store,
 }
 
+#[derive(Clone)]
 pub struct Iter<'a> {
     pub key: u16,
     inner: store::Iter<'a>,
@@ -23,6 +27,10 @@ impl Container {
     pub fn new(key: u16) -> Container {
         Container { key, store: Store::new() }
     }
+
+    pub fn full(key: u16) -> Container {
+        Container { key, store: Store::full() }
+    }
 }
 
 impl Container {
@@ -30,6 +38,11 @@ impl Container {
         self.store.len()
     }
 
+    pub fn is_empty(&self) -> bool {
+        self.store.is_empty()
+    }
+
+    #[inline]
     pub fn insert(&mut self, index: u16) -> bool {
         if self.store.insert(index) {
             self.ensure_correct_store();
@@ -40,6 +53,12 @@ impl Container {
     }
 
     pub fn insert_range(&mut self, range: RangeInclusive<u16>) -> u64 {
+        // If inserting the range will make this a bitmap by itself, do it now
+        if range.len() as u64 > ARRAY_LIMIT {
+            if let Store::Array(arr) = &self.store {
+                self.store = Store::Bitmap(arr.to_bitmap_store());
+            }
+        }
         let inserted = self.store.insert_range(range);
         self.ensure_correct_store();
         inserted
@@ -84,10 +103,48 @@ impl Container {
         result
     }
 
+    pub fn remove_smallest(&mut self, n: u64) {
+        match &self.store {
+            Store::Bitmap(bits) => {
+                if bits.len() - n <= ARRAY_LIMIT {
+                    let mut replace_array = Vec::with_capacity((bits.len() - n) as usize);
+                    replace_array.extend(bits.iter().skip(n as usize));
+                    self.store = Store::Array(store::ArrayStore::from_vec_unchecked(replace_array));
+                } else {
+                    self.store.remove_smallest(n)
+                }
+            }
+            Store::Array(_) => self.store.remove_smallest(n),
+        };
+    }
+
+    pub fn remove_biggest(&mut self, n: u64) {
+        match &self.store {
+            Store::Bitmap(bits) => {
+                if bits.len() - n <= ARRAY_LIMIT {
+                    let mut replace_array = Vec::with_capacity((bits.len() - n) as usize);
+                    replace_array.extend(bits.iter().take((bits.len() - n) as usize));
+                    self.store = Store::Array(store::ArrayStore::from_vec_unchecked(replace_array));
+                } else {
+                    self.store.remove_biggest(n)
+                }
+            }
+            Store::Array(_) => self.store.remove_biggest(n),
+        };
+    }
+
     pub fn contains(&self, index: u16) -> bool {
         self.store.contains(index)
     }
 
+    pub fn contains_range(&self, range: RangeInclusive<u16>) -> bool {
+        self.store.contains_range(range)
+    }
+
+    pub fn is_full(&self) -> bool {
+        self.store.is_full()
+    }
+
     pub fn is_disjoint(&self, other: &Self) -> bool {
         self.store.is_disjoint(&other.store)
     }
@@ -104,6 +161,7 @@ impl Container {
         self.store.min()
     }
 
+    #[inline]
     pub fn max(&self) -> Option<u16> {
         self.store.max()
     }
@@ -120,7 +178,7 @@ impl Container {
                 }
             }
             Store::Array(ref vec) => {
-                if vec.len() as u64 > ARRAY_LIMIT {
+                if vec.len() > ARRAY_LIMIT {
                     self.store = Store::Bitmap(vec.to_bitmap_store())
                 }
             }
@@ -240,11 +298,26 @@ impl IntoIterator for Container {
     }
 }
 
-impl<'a> Iterator for Iter<'a> {
+impl Iterator for Iter<'_> {
     type Item = u32;
     fn next(&mut self) -> Option<u32> {
         self.inner.next().map(|i| util::join(self.key, i))
     }
+
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        self.inner.size_hint()
+    }
+
+    fn count(self) -> usize
+    where
+        Self: Sized,
+    {
+        self.inner.count()
+    }
+
+    fn nth(&mut self, n: usize) -> Option<Self::Item> {
+        self.inner.nth(n).map(|i| util::join(self.key, i))
+    }
 }
 
 impl DoubleEndedIterator for Iter<'_> {
@@ -253,6 +326,18 @@ impl DoubleEndedIterator for Iter<'_> {
     }
 }
 
+impl ExactSizeIterator for Iter<'_> {}
+
+impl Iter<'_> {
+    pub(crate) fn advance_to(&mut self, index: u16) {
+        self.inner.advance_to(index);
+    }
+
+    pub(crate) fn advance_back_to(&mut self, index: u16) {
+        self.inner.advance_back_to(index);
+    }
+}
+
 impl fmt::Debug for Container {
     fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
         format!("Container<{:?} @ {:?}>", self.len(), self.key).fmt(formatter)
