@@ -3,11 +3,16 @@ mod vector;
 mod visitor;
 
 use crate::bitmap::store::array_store::visitor::{CardinalityCounter, VecWriter};
-use std::cmp::Ordering;
-use std::cmp::Ordering::*;
-use std::convert::{TryFrom, TryInto};
-use std::fmt::{Display, Formatter};
-use std::ops::{BitAnd, BitAndAssign, BitOr, BitXor, RangeInclusive, Sub, SubAssign};
+use core::cmp::Ordering;
+use core::cmp::Ordering::*;
+use core::fmt::{Display, Formatter};
+use core::ops::{BitAnd, BitAndAssign, BitOr, BitXor, RangeInclusive, Sub, SubAssign};
+
+#[cfg(not(feature = "std"))]
+use alloc::vec::Vec;
+
+#[cfg(not(feature = "std"))]
+use alloc::boxed::Box;
 
 use super::bitmap_store::{bit, key, BitmapStore, BITMAP_LENGTH};
 
@@ -21,6 +26,15 @@ impl ArrayStore {
         ArrayStore { vec: vec![] }
     }
 
+    pub fn with_capacity(capacity: usize) -> ArrayStore {
+        ArrayStore { vec: Vec::with_capacity(capacity) }
+    }
+
+    /// The number of total values that can be inserted without needing to reallocate.
+    pub fn capacity(&self) -> usize {
+        self.vec.capacity()
+    }
+
     ///
     /// Create a new SortedU16Vec from a given vec
     /// It is up to the caller to ensure the vec is sorted and deduplicated
@@ -38,6 +52,7 @@ impl ArrayStore {
         }
     }
 
+    #[inline]
     pub fn insert(&mut self, index: u16) -> bool {
         self.vec.binary_search(&index).map_err(|loc| self.vec.insert(loc, index)).is_err()
     }
@@ -48,16 +63,11 @@ impl ArrayStore {
 
         // Figure out the starting/ending position in the vec.
         let pos_start = self.vec.binary_search(&start).unwrap_or_else(|x| x);
-        let pos_end = self
-            .vec
-            .binary_search_by(|p| {
-                // binary search the right most position when equals
-                match p.cmp(&end) {
-                    Greater => Greater,
-                    _ => Less,
-                }
-            })
-            .unwrap_or_else(|x| x);
+        let pos_end = pos_start
+            + match self.vec[pos_start..].binary_search(&end) {
+                Ok(x) => x + 1,
+                Err(x) => x,
+            };
 
         // Overwrite the range in the middle - there's no need to take
         // into account any existing elements between start and end, as
@@ -102,24 +112,45 @@ impl ArrayStore {
 
         // Figure out the starting/ending position in the vec.
         let pos_start = self.vec.binary_search(&start).unwrap_or_else(|x| x);
-        let pos_end = self
-            .vec
-            .binary_search_by(|p| {
-                // binary search the right most position when equals
-                match p.cmp(&end) {
-                    Greater => Greater,
-                    _ => Less,
-                }
-            })
-            .unwrap_or_else(|x| x);
+        let pos_end = pos_start
+            + match self.vec[pos_start..].binary_search(&end) {
+                Ok(x) => x + 1,
+                Err(x) => x,
+            };
         self.vec.drain(pos_start..pos_end);
         (pos_end - pos_start) as u64
     }
 
+    pub fn remove_smallest(&mut self, n: u64) {
+        self.vec.rotate_left(n as usize);
+        self.vec.truncate(self.vec.len() - n as usize);
+    }
+
+    pub fn remove_biggest(&mut self, n: u64) {
+        self.vec.truncate(self.vec.len() - n as usize);
+    }
+
     pub fn contains(&self, index: u16) -> bool {
         self.vec.binary_search(&index).is_ok()
     }
 
+    pub fn contains_range(&self, range: RangeInclusive<u16>) -> bool {
+        let start = *range.start();
+        let end = *range.end();
+        let range_count = usize::from(end - start) + 1;
+        if self.vec.len() < range_count {
+            return false;
+        }
+        let start_i = match self.vec.binary_search(&start) {
+            Ok(i) => i,
+            Err(_) => return false,
+        };
+
+        // If there are `range_count` items, last item in the next range_count should be the
+        // expected end value, because this vec is sorted and has no duplicates
+        self.vec.get(start_i + range_count - 1) == Some(&end)
+    }
+
     pub fn is_disjoint(&self, other: &Self) -> bool {
         let (mut i1, mut i2) = (self.vec.iter(), other.vec.iter());
         let (mut value1, mut value2) = (i1.next(), i2.next());
@@ -163,7 +194,7 @@ impl ArrayStore {
 
     pub fn to_bitmap_store(&self) -> BitmapStore {
         let mut bits = Box::new([0; BITMAP_LENGTH]);
-        let len = self.len() as u64;
+        let len = self.len();
 
         for &index in self.iter() {
             bits[key(index)] |= 1 << bit(index);
@@ -175,10 +206,15 @@ impl ArrayStore {
         self.vec.len() as u64
     }
 
+    pub fn is_empty(&self) -> bool {
+        self.vec.is_empty()
+    }
+
     pub fn min(&self) -> Option<u16> {
         self.vec.first().copied()
     }
 
+    #[inline]
     pub fn max(&self) -> Option<u16> {
         self.vec.last().copied()
     }
@@ -194,17 +230,34 @@ impl ArrayStore {
         self.vec.get(n as usize).cloned()
     }
 
-    pub fn iter(&self) -> std::slice::Iter<u16> {
+    pub fn iter(&self) -> core::slice::Iter<u16> {
         self.vec.iter()
     }
 
-    pub fn into_iter(self) -> std::vec::IntoIter<u16> {
+    pub fn into_iter(self) -> alloc::vec::IntoIter<u16> {
         self.vec.into_iter()
     }
 
     pub fn as_slice(&self) -> &[u16] {
         &self.vec
     }
+
+    /// Retains only the elements specified by the predicate.
+    pub fn retain(&mut self, mut f: impl FnMut(u16) -> bool) {
+        // Idea to avoid branching from "Engineering Fast Indexes for Big Data
+        // Applications" talk by Daniel Lemire
+        // (https://youtu.be/1QMgGxiCFWE?t=1242).
+        let slice = self.vec.as_mut_slice();
+        let mut pos = 0;
+        for i in 0..slice.len() {
+            let val = slice[i];
+            // We want to do `slice[pos] = val` but we don't need the bounds check.
+            // SAFETY: pos is always at most i because `f(val) as usize` is at most 1.
+            unsafe { *slice.get_unchecked_mut(pos) = val }
+            pos += f(val) as usize;
+        }
+        self.vec.truncate(pos);
+    }
 }
 
 impl Default for ArrayStore {
@@ -226,7 +279,7 @@ pub enum ErrorKind {
 }
 
 impl Display for Error {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> core::fmt::Result {
         match self.kind {
             ErrorKind::Duplicate => {
                 write!(f, "Duplicate element found at index: {}", self.index)
@@ -238,6 +291,7 @@ impl Display for Error {
     }
 }
 
+#[cfg(feature = "std")]
 impl std::error::Error for Error {}
 
 impl TryFrom<Vec<u16>> for ArrayStore {
@@ -300,9 +354,9 @@ impl BitAndAssign<&Self> for ArrayStore {
         #[cfg(not(feature = "simd"))]
         {
             let mut i = 0;
-            self.vec.retain(|x| {
-                i += rhs.iter().skip(i).position(|y| y >= x).unwrap_or(rhs.vec.len());
-                rhs.vec.get(i).map_or(false, |y| x == y)
+            self.retain(|x| {
+                i += rhs.iter().skip(i).position(|y| *y >= x).unwrap_or(rhs.vec.len());
+                rhs.vec.get(i).map_or(false, |y| x == *y)
             });
         }
     }
@@ -310,7 +364,7 @@ impl BitAndAssign<&Self> for ArrayStore {
 
 impl BitAndAssign<&BitmapStore> for ArrayStore {
     fn bitand_assign(&mut self, rhs: &BitmapStore) {
-        self.vec.retain(|x| rhs.contains(*x));
+        self.retain(|x| rhs.contains(x));
     }
 }
 
@@ -339,9 +393,9 @@ impl SubAssign<&Self> for ArrayStore {
         #[cfg(not(feature = "simd"))]
         {
             let mut i = 0;
-            self.vec.retain(|x| {
-                i += rhs.iter().skip(i).position(|y| y >= x).unwrap_or(rhs.vec.len());
-                rhs.vec.get(i).map_or(true, |y| x != y)
+            self.retain(|x| {
+                i += rhs.iter().skip(i).position(|y| *y >= x).unwrap_or(rhs.vec.len());
+                rhs.vec.get(i).map_or(true, |y| x != *y)
             });
         }
     }
@@ -349,7 +403,7 @@ impl SubAssign<&Self> for ArrayStore {
 
 impl SubAssign<&BitmapStore> for ArrayStore {
     fn sub_assign(&mut self, rhs: &BitmapStore) {
-        self.vec.retain(|x| !rhs.contains(*x));
+        self.retain(|x| !rhs.contains(x));
     }
 }
 
@@ -429,6 +483,20 @@ mod tests {
         assert_eq!(into_vec(store), vec![1, 2, 4, 5, 6, 7, 8, 9]);
     }
 
+    #[test]
+    fn test_array_contains_range() {
+        let store = Store::Array(ArrayStore::from_vec_unchecked(vec![]));
+        assert!(!store.contains_range(0..=0));
+        assert!(!store.contains_range(0..=1));
+        assert!(!store.contains_range(1..=u16::MAX));
+
+        let store = Store::Array(ArrayStore::from_vec_unchecked(vec![0, 1, 2, 3, 4, 5, 100]));
+        assert!(store.contains_range(0..=0));
+        assert!(store.contains_range(0..=5));
+        assert!(!store.contains_range(0..=6));
+        assert!(store.contains_range(100..=100));
+    }
+
     #[test]
     fn test_array_insert_range_full_overlap() {
         let mut store = Store::Array(ArrayStore::from_vec_unchecked(vec![1, 2, 8, 9]));
@@ -473,7 +541,7 @@ mod tests {
 
         let mut want = vec![1, 2];
         want.extend(4..129);
-        want.extend(&[130]);
+        want.extend([130]);
 
         assert_eq!(into_vec(store), want);
     }
@@ -488,7 +556,7 @@ mod tests {
 
         let mut want = Vec::new();
         want.extend(1..129);
-        want.extend(&[130]);
+        want.extend([130]);
 
         assert_eq!(into_vec(store), want);
     }
@@ -520,4 +588,18 @@ mod tests {
 
         assert_eq!(into_vec(store), want);
     }
+
+    #[test]
+    fn test_bitmap_remove_smallest() {
+        let mut store = Store::Array(ArrayStore::from_vec_unchecked(vec![1, 2, 130, 500]));
+        store.remove_smallest(3);
+        assert_eq!(into_vec(store), vec![500]);
+    }
+
+    #[test]
+    fn test_bitmap_remove_biggest() {
+        let mut store = Store::Array(ArrayStore::from_vec_unchecked(vec![1, 2, 130, 500]));
+        store.remove_biggest(2);
+        assert_eq!(into_vec(store), vec![1, 2]);
+    }
 }
