@@ -1,10 +1,15 @@
-use std::borrow::Borrow;
-use std::cmp::Ordering;
-use std::fmt::{Display, Formatter};
-use std::ops::{BitAndAssign, BitOrAssign, BitXorAssign, RangeInclusive, SubAssign};
+use core::borrow::Borrow;
+use core::cmp::Ordering;
+use core::fmt::{Display, Formatter};
+use core::ops::{BitAndAssign, BitOrAssign, BitXorAssign, RangeInclusive, SubAssign};
 
 use super::ArrayStore;
 
+#[cfg(not(feature = "std"))]
+use alloc::boxed::Box;
+#[cfg(not(feature = "std"))]
+use alloc::vec::Vec;
+
 pub const BITMAP_LENGTH: usize = 1024;
 
 #[derive(Clone, Eq, PartialEq)]
@@ -18,6 +23,14 @@ impl BitmapStore {
         BitmapStore { len: 0, bits: Box::new([0; BITMAP_LENGTH]) }
     }
 
+    pub fn full() -> BitmapStore {
+        BitmapStore { len: (BITMAP_LENGTH as u64) * 64, bits: Box::new([u64::MAX; BITMAP_LENGTH]) }
+    }
+
+    pub fn capacity(&self) -> usize {
+        BITMAP_LENGTH * u64::BITS as usize
+    }
+
     pub fn try_from(len: u64, bits: Box<[u64; BITMAP_LENGTH]>) -> Result<BitmapStore, Error> {
         let actual_len = bits.iter().map(|v| v.count_ones() as u64).sum();
         if len != actual_len {
@@ -43,6 +56,7 @@ impl BitmapStore {
         }
     }
 
+    #[inline]
     pub fn insert(&mut self, index: u16) -> bool {
         let (key, bit) = (key(index), bit(index));
         let old_w = self.bits[key];
@@ -178,6 +192,37 @@ impl BitmapStore {
         self.bits[key(index)] & (1 << bit(index)) != 0
     }
 
+    pub fn contains_range(&self, range: RangeInclusive<u16>) -> bool {
+        let start = *range.start();
+        let end = *range.end();
+        if self.len() < u64::from(end - start) + 1 {
+            return false;
+        }
+
+        let (start_i, start_bit) = (key(start), bit(start));
+        let (end_i, end_bit) = (key(end), bit(end));
+
+        // Create a mask to exclude the first `start_bit` bits
+        // e.g. if we start at bit index 1, this will create a mask which includes all but the bit
+        // at index 0.
+        let start_mask = !((1 << start_bit) - 1);
+        // We want to create a mask which includes the end_bit, so we create a mask of
+        // `end_bit + 1` bits. `end_bit` will be between [0, 63], so we create a mask including
+        // between [1, 64] bits. For example, if the last bit is the 0th bit, we make a mask with
+        // only the 0th bit set (one bit).
+        let end_mask = (!0) >> (64 - (end_bit + 1));
+
+        match &self.bits[start_i..=end_i] {
+            [] => unreachable!(),
+            &[word] => word & (start_mask & end_mask) == (start_mask & end_mask),
+            &[first, ref rest @ .., last] => {
+                (first & start_mask) == start_mask
+                    && rest.iter().all(|&word| word == !0)
+                    && (last & end_mask) == end_mask
+            }
+        }
+    }
+
     pub fn is_disjoint(&self, other: &BitmapStore) -> bool {
         self.bits.iter().zip(other.bits.iter()).all(|(&i1, &i2)| (i1 & i2) == 0)
     }
@@ -201,6 +246,10 @@ impl BitmapStore {
         self.len
     }
 
+    pub fn is_empty(&self) -> bool {
+        self.len == 0
+    }
+
     pub fn min(&self) -> Option<u16> {
         self.bits
             .iter()
@@ -209,6 +258,7 @@ impl BitmapStore {
             .map(|(index, bit)| (index * 64 + (bit.trailing_zeros() as usize)) as u16)
     }
 
+    #[inline]
     pub fn max(&self) -> Option<u16> {
         self.bits
             .iter()
@@ -267,6 +317,57 @@ impl BitmapStore {
     pub fn as_array(&self) -> &[u64; BITMAP_LENGTH] {
         &self.bits
     }
+
+    pub fn clear(&mut self) {
+        self.bits.fill(0);
+        self.len = 0;
+    }
+
+    /// Set N bits that are currently 1 bit from the lower bit to 0.
+    pub fn remove_smallest(&mut self, mut clear_bits: u64) {
+        if self.len() < clear_bits {
+            self.clear();
+            return;
+        }
+        self.len -= clear_bits;
+        for word in self.bits.iter_mut() {
+            let count = word.count_ones() as u64;
+            if clear_bits < count {
+                for _ in 0..clear_bits {
+                    *word = *word & (*word - 1);
+                }
+                return;
+            }
+            *word = 0;
+            clear_bits -= count;
+            if clear_bits == 0 {
+                return;
+            }
+        }
+    }
+
+    /// Set N bits that are currently 1 bit from the lower bit to 0.
+    pub fn remove_biggest(&mut self, mut clear_bits: u64) {
+        if self.len() < clear_bits {
+            self.clear();
+            return;
+        }
+        self.len -= clear_bits;
+        for word in self.bits.iter_mut().rev() {
+            let count = word.count_ones() as u64;
+            if clear_bits < count {
+                for _ in 0..clear_bits {
+                    *word &= !(1 << (63 - word.leading_zeros()));
+                }
+                return;
+            }
+            *word = 0;
+            clear_bits -= count;
+            if clear_bits == 0 {
+                return;
+            }
+        }
+    }
 }
 
 // this can be done in 3 instructions on x86-64 with bmi2 with: tzcnt(pdep(1 << rank, value))
@@ -296,21 +397,24 @@ pub enum ErrorKind {
 }
 
 impl Display for Error {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> core::fmt::Result {
         match self.kind {
             ErrorKind::Cardinality { expected, actual } => {
-                write!(f, "Expected cardinality was {} but was {}", expected, actual)
+                write!(f, "Expected cardinality was {expected} but was {actual}")
             }
         }
     }
 }
 
+#[cfg(feature = "std")]
 impl std::error::Error for Error {}
 
+#[derive(Clone)]
 pub struct BitmapIter<B: Borrow<[u64; BITMAP_LENGTH]>> {
-    key: usize,
+    key: u16,
     value: u64,
-    key_back: usize,
+    key_back: u16,
+    // If key_back <= key, current back value is actually in `value`
     value_back: u64,
     bits: B,
 }
@@ -320,35 +424,117 @@ impl<B: Borrow<[u64; BITMAP_LENGTH]>> BitmapIter<B> {
         BitmapIter {
             key: 0,
             value: bits.borrow()[0],
-            key_back: BITMAP_LENGTH - 1,
+            key_back: BITMAP_LENGTH as u16 - 1,
             value_back: bits.borrow()[BITMAP_LENGTH - 1],
             bits,
         }
     }
+
+    /// Advance the iterator to the first value greater than or equal to `n`.
+    pub(crate) fn advance_to(&mut self, index: u16) {
+        let new_key = key(index) as u16;
+        let value = match new_key.cmp(&self.key) {
+            Ordering::Less => return,
+            Ordering::Equal => self.value,
+            Ordering::Greater => {
+                let bits = self.bits.borrow();
+                let cmp = new_key.cmp(&self.key_back);
+                // Match arms can be reordered, this ordering is perf sensitive
+                if cmp == Ordering::Less {
+                    // new_key is > self.key, < self.key_back, so it must be in bounds
+                    unsafe { *bits.get_unchecked(new_key as usize) }
+                } else if cmp == Ordering::Equal {
+                    self.value_back
+                } else {
+                    self.value_back = 0;
+                    return;
+                }
+            }
+        };
+        let bit = bit(index);
+        let low_bits = (1 << bit) - 1;
+
+        self.key = new_key;
+        self.value = value & !low_bits;
+    }
+
+    /// Advance the back of iterator to the first value less than or equal to `n`.
+    pub(crate) fn advance_back_to(&mut self, index: u16) {
+        let new_key = key(index) as u16;
+        let (value, dst) = match new_key.cmp(&self.key_back) {
+            Ordering::Greater => return,
+            Ordering::Equal => {
+                let dst =
+                    if self.key_back <= self.key { &mut self.value } else { &mut self.value_back };
+                (*dst, dst)
+            }
+            Ordering::Less => {
+                let bits = self.bits.borrow();
+                let cmp = new_key.cmp(&self.key);
+                // Match arms can be reordered, this ordering is perf sensitive
+                if cmp == Ordering::Greater {
+                    // new_key is > self.key, < self.key_back, so it must be in bounds
+                    let value = unsafe { *bits.get_unchecked(new_key as usize) };
+                    (value, &mut self.value_back)
+                } else if cmp == Ordering::Equal {
+                    (self.value, &mut self.value)
+                } else {
+                    (0, &mut self.value)
+                }
+            }
+        };
+        let bit = bit(index);
+        let low_bits = u64::MAX >> (64 - bit - 1);
+
+        self.key_back = new_key;
+        *dst = value & low_bits;
+    }
 }
 
 impl<B: Borrow<[u64; BITMAP_LENGTH]>> Iterator for BitmapIter<B> {
     type Item = u16;
 
     fn next(&mut self) -> Option<u16> {
-        loop {
-            if self.value == 0 {
-                self.key += 1;
-                let cmp = self.key.cmp(&self.key_back);
-                // Match arms can be reordered, this ordering is perf sensitive
-                self.value = if cmp == Ordering::Less {
-                    unsafe { *self.bits.borrow().get_unchecked(self.key) }
-                } else if cmp == Ordering::Equal {
-                    self.value_back
-                } else {
+        if self.value == 0 {
+            'get_val: {
+                if self.key >= self.key_back {
                     return None;
-                };
-                continue;
+                }
+                for key in self.key + 1..self.key_back {
+                    self.value = unsafe { *self.bits.borrow().get_unchecked(key as usize) };
+                    if self.value != 0 {
+                        self.key = key;
+                        break 'get_val;
+                    }
+                }
+                self.key = self.key_back;
+                self.value = self.value_back;
+                if self.value == 0 {
+                    return None;
+                }
             }
-            let index = self.value.trailing_zeros() as usize;
-            self.value &= self.value - 1;
-            return Some((64 * self.key + index) as u16);
         }
+        let index = self.value.trailing_zeros() as u16;
+        self.value &= self.value - 1;
+        Some(64 * self.key + index)
+    }
+
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        let mut len: u32 = self.value.count_ones();
+        if self.key < self.key_back {
+            for v in &self.bits.borrow()[self.key as usize + 1..self.key_back as usize] {
+                len += v.count_ones();
+            }
+            len += self.value_back.count_ones();
+        }
+        (len as usize, Some(len as usize))
+    }
+
+    fn count(self) -> usize
+    where
+        Self: Sized,
+    {
+        self.len()
     }
 }
 
@@ -362,17 +548,20 @@ impl<B: Borrow<[u64; BITMAP_LENGTH]>> DoubleEndedIterator for BitmapIter<B> {
                     return None;
                 }
                 self.key_back -= 1;
-                self.value_back = unsafe { *self.bits.borrow().get_unchecked(self.key_back) };
+                self.value_back =
+                    unsafe { *self.bits.borrow().get_unchecked(self.key_back as usize) };
                 continue;
             }
-            let index_from_left = value.leading_zeros() as usize;
+            let index_from_left = value.leading_zeros() as u16;
             let index = 63 - index_from_left;
             *value &= !(1 << index);
-            return Some((64 * self.key_back + index) as u16);
+            return Some(64 * self.key_back + index);
         }
     }
 }
 
+impl<B: Borrow<[u64; BITMAP_LENGTH]>> ExactSizeIterator for BitmapIter<B> {}
+
 #[inline]
 pub fn key(index: u16) -> usize {
     index as usize / 64
@@ -455,3 +644,38 @@ impl BitXorAssign<&ArrayStore> for BitmapStore {
         self.len = len as u64;
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_bitmap_remove_smallest() {
+        let mut store = BitmapStore::new();
+        let range = RangeInclusive::new(1, 3);
+        store.insert_range(range);
+        let range_second = RangeInclusive::new(5, 65535);
+        // store.bits[0] = 0b1111111111111111111111111111111111111111111111111111111111101110
+        store.insert_range(range_second);
+        store.remove_smallest(2);
+        assert_eq!(
+            store.bits[0],
+            0b1111111111111111111111111111111111111111111111111111111111101000
+        );
+    }
+
+    #[test]
+    fn test_bitmap_remove_biggest() {
+        let mut store = BitmapStore::new();
+        let range = RangeInclusive::new(1, 3);
+        store.insert_range(range);
+        let range_second = RangeInclusive::new(5, 65535);
+        // store.bits[1023] = 0b1111111111111111111111111111111111111111111111111111111111111111
+        store.insert_range(range_second);
+        store.remove_biggest(2);
+        assert_eq!(
+            store.bits[1023],
+            0b11111111111111111111111111111111111111111111111111111111111111
+        );
+    }
+}
